% Estimation of the Hurst function H of a MBM (Multifractional Brownian Motion) with a quadratic variation estimator
% (QV) and the IR statistic (IR)
%
% Input: eta=regularity of the function H (eta=0 -> analytic case)
%        n=length of the sample
%        tot=number of replications
%
% Output: HQV=vector of QV estimations
%         HIR=vector of IR estimations
%         VHQV, MISEQV=variance and MISE of QV estimations 
%         VHIR, MISEIR=variance and MISE of IR estimations 
%
% 8 figures are also drawn for 8 different window length
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [HQV,HIR,VHQV,VHIR,MISEQV,MISEIR]=VariaIR_MBM(eta,n,tot)
p=0;

fBm=0;
fBm1=0;
%n=6000;

R2=0;
%eta=1.8;

eta2=eta;
if eta==1
   eta=1.0001;
end

if eta>1
   eta2=eta-1;
end
if eta>0
  [fGn1,fBm1] = genFBMJFCDon(n,eta2); % Generation of the function H as a FBM
   if eta>1
      fBm1=cumsum(fBm1);
   end
   fBm=fBm1(1:n);
   ymax=max(fBm);
   ymin=min(fBm);
   H1(1:n)=min(eta,1)*(0.1+0.8*(fBm-ymin)/(ymax-ymin)); % Definition of H
end 

%pause


if eta==0
    H1(1:n)=0.6;%0.1+0.8*(1-((1:n)/n.*sin(10*(1:n)/n)).^2); % Case when H is analytic
end
%plot(H1)

q=0;
M=5;
nbt=200;

HH7=0;
Hest=0;
Hest2=0;
R2=0;
MIQV=0;
MIIR=0;
HQV=0;
VHQV=0;
HQVP=0;
HQVM=0;
HIR=0;
HIRP=0;
HIRM=0;
VHIR=0;
qal=0;

% Computation of a path of a MBM 
SS=0;
sigma=0;
jj=1:n;
KKjj=2*H1(jj).*gamma(2*H1(jj)).*sin(pi*2*H1(jj)/2)/pi;
tic
for j=1:n
    KKj=2*H1(j).*gamma(2*H1(j)).*sin(pi*2*H1(j)/2)/pi;
    KKjjj=(H1(j)+H1(jj)).*gamma((H1(j)+H1(jj))).*sin(pi*(H1(j)+H1(jj))/2)/pi;
    sigma(j,jj)=0.5*sqrt(KKj.*KKjj)./KKjjj.*((j/n).^(H1(j)+H1(jj))+(jj/n).^(H1(j)+H1(jj))-abs((jj-j)/n).^(H1(j)+H1(jj)));
end
SS=chol(sigma,'lower'); % Generation of a MBM
toc

% Estimation of the function H following the length of the window $n^{1-alpha}$

qal=0;
valalpha=[0.15,0.2,0.25,0.30,0.35,0.40,0.45,0.5]; % Choice of the local window of length $n^{1-alpha}$
lal=length(valalpha);
for rep=1:tot
      tic 
      mfBm=0;
      mfGn=0;
      Z=randn(n,1);
      mfBm=SS*Z;
      mfGn=mfBm(2:n)'-mfBm(1:(n-1))';
      %plot(mfGn)
      %pause
      S=0;
      for alpha=valalpha
            qal=qal+1;
            q=0;
            for t=fix((2+n^(1-alpha))):fix((n/nbt)):(n-2*M-fix(n^(1-alpha)))   
                q=q+1;
                pdep=fix(t-n^(1-alpha));
                pfin=fix(t+n^(1-alpha));
   
                for m=1:M
                    p=pdep:m:pfin;
                    S(m)=mean((mfBm(2*m+p)-2*mfBm(m+p)+mfBm(p)).^2);
                    A(m)=log(m)-mean(log(1:M));
                end
                p=0;
                Hest(q,qal)=A*(log(S))'/(2*sum(A.^2)); % Computation of the QV estimator
   
                p=pdep:pfin;
                R2(q,qal)=mean(abs(mfGn(3+p)-mfGn(1+p))./(abs(mfGn(3+p)-mfGn(2+p))+abs(mfGn(2+p)-mfGn(1+p))));
                Hest2(q,qal)=HestIR(R2(q,qal),2); % Computation of the IR estimator
            end
      end
      toc
end

qal=0;
HH7=0;

HH7=H1(1:n);
for alpha=valalpha
    qal=qal+1;
    indt=0;
    longt=(n-2*M-fix(n^(1-alpha)))-fix((2+n^(1-alpha)))+1;
    for t=fix((2+n^(1-alpha))):fix((n/nbt)):(n-2*M-fix(n^(1-alpha)))   
        indt=indt+1;
        HQV(indt,qal)=mean(Hest(indt,qal+lal*(0:(tot-1))));
        VQV(indt,qal)=std(Hest(indt,qal+lal*(0:(tot-1))));
        MIQV(indt,qal)=mean((Hest(indt,qal+lal*(0:(tot-1)))-H1(t)).^2);
        HIR(indt,qal)=mean(Hest2(indt,qal+lal*(0:(tot-1))));
        VIR(indt,qal)=std(Hest2(indt,qal+lal*(0:(tot-1))));
        MIIR(indt,qal)=mean((Hest2(indt,qal+lal*(0:(tot-1)))-H1(t)).^2);
    end
    VHQV(qal)=mean(VQV(:,qal));
    VHIR(qal)=mean(VIR(:,qal));
    MISEQV(qal)=sqrt(sum(MIQV(:,qal))/nbt*n/longt);
    MISEIR(qal)=sqrt(sum(MIIR(:,qal))/nbt*n/longt);
    figure(qal)
    plot((1:n)/n,HH7)
    hold on
    indx=((1+n^(1-alpha)):(n/nbt):(n-2*M-fix(n^(1-alpha))))/n;
    longx=length(indx);
    plot(indx,HQV(1:longx,qal),'g')
    plot(indx,HIR(1:longx,qal),'r')
    xlabel('time')
    ylabel('H')
    legend('Function H','Estimator HQV','Estimator HIR')
    title(alpha)
    hold off

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Computation of the IR estimator
%
% Input : R=IR statistic
%         p=1 or 2 order of the IR statistic
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function HDon=HestIR(R,p)

H=0.0001:0.0001:0.9999;
if p==1
    r=2.^(2*H-1)-1;
end
if p==2
    r=(-3.^(2*H)+2.^(2*H+2)-7)./(8-2.^(2*H+1));
end
Lambda=1/pi*acos(-r)+sqrt(1-r.^2)./(pi*(1-r)).*log(2./(r+1));
HDon=sum(R>Lambda)/10000;
   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                                   genFBMJFCDon
%
%  function generating a time series from a fractional Gaussian noise (and the 
%  associated fractional Brownian motion) from the circulant matrix procedure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Input	    	n    : length of the time series (positive integer)
%    		        H    : self-similarity parameter (0<H<1)
%
%  Output	        fGN  : a vector (fGn(1),..,fGn(n)) of a path of a fractional 
%                          Gaussian noise of parameter H and variance 1
%                   fBm  : a vector (fBm(1),..,fBm(n)) of a path of a fractional
% 			               Brownian motion such that fGn(i)=fBm(i+1)-fBm(i) for 
%                          i=1,2,..,n
%
%  Example          [fGn,fBm]=genFBMJFCDon(1000,0.3)
%
%		Created by J-F. Coeurjolly (Universit de Grenoble, France) + Bardet, 2008
%                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [fGn,fBm]=genFBMJFCDon(n,H)

m=2^nextpow2(2*(n-1));	    	% research of the power of two greater than 2*(n-1)
u=covFGNJFCDon(m/2,H); 	        % u=c(0),c(1),...c(m/2)
ligneC=[u fliplr(u(2:(m/2)))];    % [[c(0),c(1),...c(m/2)],[c(m/2-1),...,c(1)]]
eg=real(fft(ligneC))  ;
% eigenvalues of the Circulant Matrix 
%ar		=  randn(1,fix(m/2 + 1));  	
%ai		=  randn(1,fix(m/2 + 1));			
%ar(1) 		=  sqrt(2) * ar(1);
%ar(fix(m/2 + 1)) 	=  sqrt(2) * ar(fix(m/2 + 1));
%ai(1) 	=  0;
%ai(fix(m/2 + 1)) 	=  0;
%ar		= [ar(1:fix(m/2 + 1))  ar(fix(m/2):-1:2)];
%aic 	= - ai;
%ai		= [ai(1:fix(m/2 + 1))  aic(fix(m/2):-1:2)];
%a		= (1/(sqrt(2 * m))) *real(fft(sqrt(eg) .*(ar+i*ai))); % simulation of a fGn 
                                                            % of length m

ar		=  randn(1,m)/sqrt(2);  	
ai		=  randn(1,m)/sqrt(2);			
a= real(ifft(sqrt(eg) .*(ar+i*ai)))*sqrt(m)*2^H;%ar		= [ar(1:fix(m/2 + 1))  ar(fix(m/2):-1:2)];
%aic 	= - ai;
%ai		= [ai(1:fix(m/2 + 1))  aic(fix(m/2):-1:2)];
%a		= (1/(sqrt(2 * m))) *real(fft(sqrt(eg) .*(ar+i*ai))); % simulation of a fGn 
if (H==1/2)
  	variance = 1;
else
   variance = gamma(1-2*H)*cos(pi*H)/(pi*H);
   %variance = H*gamma(2*H)*sin(pi*H)/pi;
end
sqrt(variance) ;                                                          % of length m
n2=fix(n/2);
s=std(a);
fGn     = a(n2:(n2+n-1))/(s*n^H);               % simulation of a fGn of length n 
fBm	  = cumsum(fGn);          % simulation of a fBm
std(fGn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cFGN=covFGNJFCDon(n,H)

k =  0:(n);
H2 = 2 * H;
cFGN = (abs((k - 1)).^H2 - 2 * abs(k).^H2 + abs((k + 1)).^H2)/2;
